/*
   -------------------------------------------------------------------
   EmonESP Serial to Emoncms gateway
   -------------------------------------------------------------------
   Adaptation of Chris Howells OpenEVSE ESP Wifi
   by Trystan Lea, Glyn Hudson, OpenEnergyMonitor

   Modified to use with the CircuitSetup.us Split Phase Energy Meter by jdeglavina

   All adaptation GNU General Public License as below.

   -------------------------------------------------------------------

   This file is part of OpenEnergyMonitor.org project.
   EmonESP is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
   EmonESP is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with EmonESP; see the file COPYING.  If not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef _EMONESP_CONFIG_H
#define _EMONESP_CONFIG_H

#define ATM90_MOSI  (33)
#define ATM90_MISO  (34)
#define ATM90_SCK   (7)

#include "energy_meter.h"
#include <Arduino.h>
#include <ESPAsyncWebServer.h>
#include <Preferences.h> // Move EEPROM to Preferences

// Global config varables

// Wifi Network Strings
extern String esid;
extern String epass;

// Web server authentication (leave blank for none)
extern String www_username;
extern String www_password;

// MQTT Settings
extern String mqtt_server;
extern String mqtt_topic;
extern String mqtt_user;
extern String mqtt_pass;
extern String mqtt_feed_prefix;

// Calibration Settings for CircuitSetup 6 channel energy meter
extern unsigned short voltage_cal;
extern unsigned short voltage2_cal;
extern unsigned short freq_cal;
extern unsigned short gain_cal[NUM_BOARDS];
extern String ct_name[NUM_CHANNELS];
extern unsigned short ct_cal[NUM_CHANNELS];
extern float cur_mul[NUM_CHANNELS];
extern float pow_mul[NUM_CHANNELS];

struct config_flags_t
{
  bool mqtt_json:1;
};

extern config_flags_t config_flags;

// -------------------------------------------------------------------
// Load saved settings
// -------------------------------------------------------------------
extern void config_load_settings();

// -------------------------------------------------------------------
// Save the MQTT broker details
// -------------------------------------------------------------------
extern void config_save_mqtt(String server, String topic, String prefix, String user, String pass, bool json);

// -------------------------------------------------------------------
// Save the Calibration details
// -------------------------------------------------------------------
extern void config_save_cal(AsyncWebServerRequest * request);
// -------------------------------------------------------------------
// Save the admin/web interface details
// -------------------------------------------------------------------
extern void config_save_admin(String user, String pass);

// -------------------------------------------------------------------
// Save the Wifi details
// -------------------------------------------------------------------
extern void config_save_wifi(String qsid, String qpass);

// -------------------------------------------------------------------
// Reset the config back to defaults
// -------------------------------------------------------------------
extern void config_reset();

// -------------------------------------------------------------------
// Reset the Wifi config only
// -------------------------------------------------------------------
extern void wifi_config_reset();

#endif // _EMONESP_CONFIG_H
