#include "config.h"
#include "lora_sender.h"
#include <SPI.h>
#include <RadioLib.h>

// counter to keep track of transmitted packets
int count = 0;

volatile bool transmittedFlag = true;

int transmissionState = RADIOLIB_ERR_NONE;

SPIClass fspi_bus(HSPI);
SPISettings spiSettings(2000000, MSBFIRST, SPI_MODE0);
SX1262 radio = new Module(LORA_NSS, LORA_DIO1, LORA_NRST, LORA_BUSY, fspi_bus, spiSettings);

void lora_sender_setup() {
    fspi_bus.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_NSS);
    Serial.println("Setting LoRa Sender");
    int transmissionState = RADIOLIB_ERR_NONE;
    
    Serial.print(F("[SX1262] Initializing ... "));
    int state = radio.begin(869.0,500,12,5,0x34,21,20);
    if (state == RADIOLIB_ERR_NONE) {
        Serial.println(F("success!"));
    } else {
        Serial.print(F("failed, code "));
        Serial.println(state);
    }

    radio.setPacketSentAction(setFlag);
}

// this function is called when a complete packet
// is transmitted by the module
// IMPORTANT: this function MUST be 'void' type
//            and MUST NOT have any arguments!
#if defined(ESP8266) || defined(ESP32)
  ICACHE_RAM_ATTR
#endif
void setFlag(void) {
  // we sent a packet, set the flag
  transmittedFlag = true;
  Serial.println(F("Transmission flag set to true"));
}

void lora_sender_loop() {
    if(transmittedFlag) {

    if (transmissionState == RADIOLIB_ERR_NONE) {
      // packet was successfully sent
      Serial.println(F("transmission finished!"));

      // NOTE: when using interrupt-driven transmit method,
      //       it is not possible to automatically measure
      //       transmission data rate using getDataRate()

    } else {
      Serial.print(F("failed, code "));
      Serial.println(transmissionState);

    }

    // clean up after transmission is finished
    // this will ensure transmitter is disabled,
    // RF switch is powered down etc.
    radio.finishTransmit();
  }
}

void lora_sender_message(JsonDocument& doc) {
//    if(transmittedFlag) {
//      transmittedFlag = false;
      char msgpack[256];
      Serial.println(F("Sending packet"));
      doc["hostname"] = String("powerfly-") + String((uint32_t)(ESP.getEfuseMac() >> 16), HEX);
      serializeJsonPretty(doc,Serial);
      Serial.println("");
      serializeJson(doc, msgpack);
      int state = radio.transmit(msgpack);
      if (state == RADIOLIB_ERR_NONE) {
        // the packet was successfully transmitted
        Serial.println(F("success!"));

        // print measured data rate
        Serial.print(F("[SX1262] Datarate:\t"));
        Serial.print(radio.getDataRate());
        Serial.println(F(" bps"));

      } else if (state == RADIOLIB_ERR_PACKET_TOO_LONG) {
        // the supplied packet was longer than 256 bytes
        Serial.println(F("too long!"));

      } else if (state == RADIOLIB_ERR_TX_TIMEOUT) {
        // timeout occured while transmitting packet
        Serial.println(F("timeout!"));
      } else {
        // some other error occurred
        Serial.print(F("failed, code "));
        Serial.println(state);

      }
      delay(3000);
}
